/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is PSU Calendar Server Code, released March 3rd, 2003.
 *
 * The Initial Developer of the Original Code is
 * Penn State CSE 420w Mozilla Project
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):  Benjamin Nave <bnave@psu.edu>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include <nsIServiceManager.h>
#include "psuCalServerSunCDEImpl.h"
#include "nsIComponentRegistrar.h"

int main(void)
{
    nsresult rv;
    char *user;
    char *uri;

    nsCOMPtr<nsIServiceManager> servMan;
    NS_InitXPCOM2(getter_AddRefs(servMan), nsnull, nsnull);
    nsCOMPtr<nsIComponentRegistrar> registrar = do_QueryInterface(servMan);
    NS_ASSERTION(registrar, "Null nsIComponentRegistrar");
    registrar->AutoRegister(nsnull);

    // Create an instance of our component
    nsCOMPtr <psuICalServer> mysample = do_CreateInstance(PSU_SUNCDE_CONTRACTID, &rv);
    if (NS_FAILED(rv))
    {
        printf("ERROR: Cannot create instance of component " PSU_SUNCDE_CONTRACTID " [%x].\n"
               "Debugging hint:\n"
               "\tsetenv NSPR_LOG_MODULES nsComponentManager:5\n"
               "\tsetenv NSPR_LOG_FILE xpcom.log\n"
               "\t./TestPSUCalSunCDE\n"
               "\t<check the contents for xpcom.log for possible cause of error>.\n",
               rv);
        return -2;
    }

    user = getenv("USER");
    if(!user)
      {
        user = "source";
      }

    uri = (char *)malloc( (strlen("suncal://stewie.cede.psu.edu/") + strlen(user) + 1) * sizeof(char) );
    sprintf( uri, "%s%s", "suncal://stewie.cede.psu.edu/", user );

    rv = mysample->SetServer( uri );
    if ( NS_FAILED(rv) )
      {
        printf("ERROR: Calling psuCalServerSunCDEImpl::SetServer() [%X]\n", rv);
      }

    free( uri );

    rv = mysample->Test();

    if ( NS_FAILED(rv) )
    {
        printf("ERROR: Calling psuCalServerSunCDEImpl::Test()\n");
        return 0;
    }

    printf("Test finished\n");

    // Shutdown XPCOM
    NS_ShutdownXPCOM(nsnull);
    return 0;
}
