/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is PSU Calendar Server Code, released March 3rd, 2003.
 *
 * The Initial Developer of the Original Code is
 * Penn State CSE 420w Mozilla Project
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):  Daniel Bungert <drb@debian.org>
 *                  Thomas Keiser <tkeiser@cede.psu.edu>
 *                  Benjamin Nave <bnave@psu.edu>
 *                  John Pendzick <jnp117@psu.edu>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


#include "psuSunCalEvent.h"
#include "psuCalServerSunCDEImpl.h"
#include "nsMemory.h"
#include "nsError.h"
#include "nsString.h"
#include <stdlib.h>
#include <prdtoa.h>

psuSunCalEvent::psuSunCalEvent()
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::psuSunCalEvent() \n");
#endif
  init();
}


psuSunCalEvent::psuSunCalEvent(oeIICalEvent *oeIEvent, PRBool adding)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::psuSunCalEvent(oeIICalEvent *) \n");
#endif
  if(init() == NS_OK)
    {
      OeIICalEventToSunCalEvent(oeIEvent, adding);
    }
}

psuSunCalEvent::psuSunCalEvent( CSA_attribute *attrList, 
				CSA_uint32 numAttr, 
				PRBool nsMem )
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::psuSunCalEvent(CSA_attribute *, CSA_uint32) \n");
#endif

  if ( NS_SUCCEEDED( init() ) )
    {
      SetSunCalEvent( attrList, numAttr, nsMem );
    }
}

NS_IMETHODIMP
psuSunCalEvent::init()
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::init() \n");
#endif
  m_Entry = nsnull;
  m_AttrList = nsnull;
  m_NumAttr = 0;
  m_MemAttr = PR_TRUE;
  m_Addable = PR_FALSE;
  RETURN( NS_OK );
}

psuSunCalEvent::~psuSunCalEvent()
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::~psuSunCalEvent() \n");
#endif
  csaEntryAttrDelete();
}
  
NS_IMETHODIMP 
psuSunCalEvent::csaEntryAttrDelete()
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::csaEntryAttrDelete()\n", m_NumAttr);
#endif 
  if(m_MemAttr)
    {
      while((m_NumAttr > 0) && (m_AttrList != nsnull))
	{
	  m_NumAttr--;
	  char *name = m_AttrList[m_NumAttr].name;
	  CSAFreeAttrValue(m_AttrList[m_NumAttr].value);
	  CSAFreeExtensions(m_AttrList[m_NumAttr].attribute_extensions);
	}
      if(m_AttrList != nsnull)
	{
	  nsMemory::Free(m_AttrList);
	  m_AttrList = nsnull;
	}
      if(m_Entry != nsnull)
	{
	  csa_free((void*)m_Entry);
	  m_Entry = nsnull;
	}
    }
  else
    {
      if(m_AttrList != nsnull)
	{
	  csa_free(m_AttrList);
	  m_AttrList = nsnull;
	  m_NumAttr = 0;
	}
      if(m_Entry != nsnull)
	{
	  csa_free((void*)m_Entry);
	  m_Entry = nsnull;
	}
    }
  RETURN( NS_OK );
}

NS_IMETHODIMP
psuSunCalEvent::CSAFreeAttrValue(CSA_attribute_value *value)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DeleteAttrValue \n");
#endif
  if(value != nsnull)
    {          
      /** Handling the union */
      switch (value->type)
	{
	case CSA_VALUE_STRING:
	  if(value->item.string_value != nsnull)
	    {
	      nsMemory::Free(value->item.string_value);
	      value->item.string_value = nsnull;
	    }
	  break;
	case CSA_VALUE_DATE_TIME:
	  if(value->item.date_time_value != nsnull)
	    {
	      nsMemory::Free(value->item.date_time_value);
	      value->item.date_time_value = nsnull;
	    }
	  break;
	case CSA_VALUE_DATE_TIME_RANGE:
	  if(value->item.date_time_range_value != nsnull)
	    {
	      nsMemory::Free(value->item.date_time_range_value);
	      value->item.date_time_range_value = nsnull;
	    }
	  break;
	case CSA_VALUE_TIME_DURATION:
	  if(value->item.time_duration_value != nsnull)
	    {
	      nsMemory::Free(value->item.time_duration_value);
	      value->item.time_duration_value = nsnull;
	    }
	  break;	
	case CSA_VALUE_ACCESS_LIST:
	  if(value->item.access_list_value != nsnull)
	    {
	      nsMemory::Free(value->item.access_list_value);
	      value->item.access_list_value = nsnull;
	    }
	  break;
	case CSA_VALUE_ATTENDEE_LIST:
	  if(value->item.attendee_list_value != nsnull)
	    {
	      nsMemory::Free(value->item.attendee_list_value);
	      value->item.attendee_list_value = nsnull;
	    }
	  break;
	case CSA_VALUE_DATE_TIME_LIST:
	  if(value->item.date_time_list_value != nsnull)
	    {
	      nsMemory::Free(value->item.date_time_list_value);
	      value->item.date_time_list_value =  nsnull;
	    }
	  break;
	case CSA_VALUE_REMINDER:
	  if(value->item.reminder_value != nsnull)
	    {
	      CSAFreeReminder(value->item.reminder_value);
	    }
	  break;
	case CSA_VALUE_OPAQUE_DATA:
	  CSAFreeOpaqueData(value->item.opaque_data_value);
	  break;
	defualt:
	  break;
	}
    }
  nsMemory::Free(value);
  value = nsnull;
RETURN( NS_OK );
}

NS_IMETHODIMP
psuSunCalEvent::CSAFreeExtensions(CSA_extension *exten)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::CSAFreeExtensions\n");
#endif
  if(exten != nsnull)
    {
      if(exten->item_reference != nsnull)
	{
	  nsMemory::Free(exten->item_reference);
	  exten->item_reference = nsnull;
	}
      nsMemory::Free(exten);
      exten =  nsnull;
    }
  RETURN( NS_OK );
}

NS_IMETHODIMP
psuSunCalEvent::CSAFreeReminder(CSA_reminder *rem)
{
#ifdef PSUCAL_DEBUG
  printf("psuSUnCalEvent::CSAFreeReminder()\n");
#endif

  if(rem != nsnull)
    {
      if(rem->lead_time != nsnull)
	{
	  nsMemory::Free(rem->lead_time);
	  rem->lead_time = nsnull;
	}
      if(rem->snooze_time != nsnull)
	{
	  nsMemory::Free(rem->snooze_time);
	  rem->snooze_time = nsnull;
	}
      if(rem->reminder_data.data != nsnull)
	{
	  nsMemory::Free(rem->reminder_data.data);
	  rem->reminder_data.data =  nsnull;
	}
    }
  RETURN( NS_OK );
}

NS_IMETHODIMP
psuSunCalEvent::CSAFreeOpaqueData(CSA_opaque_data *opdata)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::CSAFreeOpaqueData()\n");
#endif
  if(opdata != nsnull)
    {
      if(opdata->data != nsnull)
	{
	  nsMemory::Free(opdata->data);
	  opdata->data = nsnull;
	}
      nsMemory::Free(opdata);
      opdata = nsnull;
    }
  RETURN( NS_OK );
}

NS_IMETHODIMP 
psuSunCalEvent::GetOeIICalEvent(oeIICalEvent **oeIEvent)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::GetOeIICalEvent() \n");
#endif
  if( oeIEvent == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }
  RETURN( SunCalEventToOeIICalEvent( oeIEvent ) );
}

NS_IMETHODIMP 
psuSunCalEvent::GetSunCalEvent(CSA_attribute ** const attrs,
			       CSA_uint32 * const numOfAttr)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::GetSunCalEvent \n");
#endif
  (*attrs) = m_AttrList;
  (*numOfAttr) = m_NumAttr;
  RETURN( NS_OK );
}

NS_IMETHODIMP 
psuSunCalEvent::SetSunCalHandle(CSA_entry_handle newHandle)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::SetSunCalHandle() \n");
#endif
  m_Entry = newHandle;
  RETURN( NS_OK );
}

NS_IMETHODIMP 
psuSunCalEvent::GetSunCalHandle(CSA_entry_handle *returnHandle)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::GetSunCalHandle() \n");
#endif
  (*returnHandle) = m_Entry;
  RETURN( NS_OK );
}

NS_IMETHODIMP 
psuSunCalEvent::SetSunCalEvent(CSA_attribute *newAttr, 
			       CSA_uint32 newAttrLength,
			       PRBool nsMem)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::SetSunCalEvent() \n");
#endif
  m_AttrList = newAttr;
  m_NumAttr = newAttrLength;
  m_MemAttr = nsMem;
  RETURN( NS_OK );
}

NS_IMETHODIMP 
psuSunCalEvent::SetSunCalEvent(oeIICalEvent *event, PRBool adding)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::SetSunCalEvent() \n");
#endif
  RETURN( OeIICalEventToSunCalEvent(event, adding) );
}

NS_IMETHODIMP 
psuSunCalEvent::OeIICalEventToSunCalEvent(oeIICalEvent *oeIIevent, 
					  PRBool adding)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::OeIICalEventToSunCalEvent() \n");
#endif
  m_MemAttr = PR_TRUE;
  CSA_attribute_value *attr_val; 
  if(oeIIevent == nsnull)
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }
  csaEntryAttrDelete();
  m_NumAttr = 0;

  /* Allocating memory to handle the CSA_attribute */
  m_AttrList = (CSA_attribute*)nsMemory::
    Alloc(sizeof(CSA_attribute) * SUNCAL_MAX_ATTRIBE);
  
  oeIDateTime *tmpTime;
  nsresult retCode;
  /* Dealing with start, BOTH TYPES OF EVENTS MUST HAVE A START DATE */
  if((oeIIevent->GetStart(&tmpTime)) ==  NS_OK)
  {
    char *startbuf;
    /* conversion of Start Date to iso8601 string necessary for Suncal */
    retCode = OeDateTimeToIso8601(&startbuf, tmpTime);
    if(retCode != NS_OK)
      {
#ifdef PSUCAL_DEBUG
	printf("psuSunCalEvent::OeIICalEventToSunCalEvent(): ");
	printf("Failed to convert iso8601\n"); 
#endif
	RETURN( retCode );
      }
    m_AttrList[m_NumAttr].name = CSA_ENTRY_ATTR_START_DATE;
    attr_val = (CSA_attribute_value*)nsMemory::
      Alloc(sizeof(CSA_attribute_value));
    if(attr_val == nsnull)
      {
	RETURN( NS_ERROR_OUT_OF_MEMORY );
      }
    m_AttrList[m_NumAttr].value = attr_val;
    m_AttrList[m_NumAttr].attribute_extensions = nsnull;	     
    attr_val->type = CSA_VALUE_DATE_TIME;
    attr_val->item.date_time_value = startbuf;
    m_NumAttr++;
  }
  else  /* if there is  no start date for event then we don't continue */
    {
#ifdef PSUCAL_DEBUG
      printf("psuCalServerImpl::OeIICalEventToSunCalEventy() : "); 
      printf("No Starting time attribute in the oeIICalEvent\n");
#endif
      RETURN( NS_ERROR_CAL_SERVER_INVALID_ATTRIBUTE );
    }
  /* setup for CSA_ENTRY_ATTR_TYPE, sun attribute MUST HAVE IT 
   * for  adding events, don't have it modify */
  if(adding)
    {
      m_AttrList[m_NumAttr].name = CSA_ENTRY_ATTR_TYPE;
      attr_val = (CSA_attribute_value*)nsMemory::
	Alloc(sizeof(CSA_attribute_value));
      if(attr_val == nsnull)
	{
	  RETURN( NS_ERROR_OUT_OF_MEMORY );
	}
      m_AttrList[m_NumAttr].value = attr_val;
      m_AttrList[m_NumAttr].attribute_extensions = nsnull;
      attr_val->type = CSA_VALUE_UINT32;
      attr_val->item.sint32_value = CSA_TYPE_EVENT;
      m_Addable = PR_TRUE;
      m_NumAttr++;
    }
  /*Is the Event public or private 
    if its private then set, the defualt for cde is public */
  PRBool priv;
  retCode = oeIIevent->GetPrivateEvent(&priv);
  if(retCode == NS_OK && priv)
    {
      m_AttrList[m_NumAttr].name = CSA_ENTRY_ATTR_CLASSIFICATION;
      attr_val = (CSA_attribute_value*)nsMemory::
	Alloc(sizeof(CSA_attribute_value));
      if(attr_val == nsnull)
	{
	  RETURN( NS_ERROR_OUT_OF_MEMORY );
	}
      m_AttrList[m_NumAttr].value = attr_val;
      m_AttrList[m_NumAttr].attribute_extensions = nsnull;
      attr_val->type = CSA_VALUE_UINT32;
      attr_val->item.sint32_value = CSA_CLASS_CONFIDENTIAL;
      m_NumAttr++;
    }

  /* Dealing with end time, its optional */
  if((retCode = oeIIevent->GetEnd(&tmpTime)) ==  NS_OK)
    {
      char *endbuf;
      retCode = OeDateTimeToIso8601(&endbuf, tmpTime);
      /* if we  fail to convert the end time we move on */
      if(retCode == NS_OK)
	{
	  m_AttrList[m_NumAttr].name = CSA_ENTRY_ATTR_END_DATE;
	  attr_val = (CSA_attribute_value*)nsMemory::
	    Alloc(sizeof(CSA_attribute_value));
	  if(attr_val == nsnull)
	    {
	      RETURN( NS_ERROR_OUT_OF_MEMORY );
	    }
	  m_AttrList[m_NumAttr].value = attr_val;
	  m_AttrList[m_NumAttr].attribute_extensions = nsnull;
	  attr_val->type = CSA_VALUE_DATE_TIME;
	  attr_val->item.date_time_value = endbuf;
	  m_NumAttr++;
	}
    }
  /* Dealing with making the description */
  char *descriptionBuf;
  retCode = MakeDescription(&descriptionBuf, oeIIevent);
  if(retCode == NS_OK)
    {
      m_AttrList[m_NumAttr].name = CSA_ENTRY_ATTR_SUMMARY;
      attr_val = (CSA_attribute_value*)nsMemory::
	Alloc(sizeof(CSA_attribute_value));
      if(attr_val == nsnull)
	{
	  RETURN( NS_ERROR_OUT_OF_MEMORY );
	}
      m_AttrList[m_NumAttr].value = attr_val;
      m_AttrList[m_NumAttr].attribute_extensions = nsnull;
      attr_val->type = CSA_VALUE_STRING;
      attr_val->item.string_value = descriptionBuf;
      m_AttrList[m_NumAttr].attribute_extensions = nsnull;
      m_NumAttr++;
    }
  /* Doing repeat Event Stuff, if any */
  PRBool recur;
  retCode = oeIIevent->GetRecur(&recur);
  if((retCode == NS_OK) && recur)
    {
      char *recurString;
      retCode = CreateRecurenceString(&recurString, oeIIevent);
      if(retCode == NS_OK)
	{
	  m_AttrList[m_NumAttr].name = CSA_ENTRY_ATTR_RECURRENCE_RULE;
	  attr_val = (CSA_attribute_value*)nsMemory::
	    Alloc(sizeof(CSA_attribute_value));
	  if(attr_val == nsnull)
	    {
	      RETURN( NS_ERROR_OUT_OF_MEMORY );
	    }
	  m_AttrList[m_NumAttr].value = attr_val;
	  m_AttrList[m_NumAttr].attribute_extensions = nsnull;
	  attr_val->type =  CSA_VALUE_STRING;
	  attr_val->item.string_value = recurString;
	  m_NumAttr++;
	}
    }
  /* dealing with email Alerts, if any */ 
  PRBool alarm;
  retCode = oeIIevent->GetAlarm(&alarm);
  if((retCode == NS_OK) && alarm)
    {
      CSA_reminder *emailReminder;
     
      m_AttrList[m_NumAttr].name = CSA_ENTRY_ATTR_MAIL_REMINDER;
      attr_val = (CSA_attribute_value*)nsMemory::
	Alloc(sizeof(CSA_attribute_value));	
      if(attr_val == nsnull)
	{
	  RETURN( NS_ERROR_OUT_OF_MEMORY );
	}
      m_AttrList[m_NumAttr].value = attr_val;
      m_AttrList[m_NumAttr].attribute_extensions = nsnull;
      attr_val->type = CSA_VALUE_REMINDER;
      emailReminder = (CSA_reminder*)nsMemory::Alloc(sizeof(CSA_reminder));
      if((emailReminder ==  nsnull))
	{
	  RETURN( NS_ERROR_OUT_OF_MEMORY );
	}
      attr_val->item.reminder_value = emailReminder;
      memset((void*)emailReminder, nsnull, sizeof(CSA_reminder));
      if(CreateEmailReminder(emailReminder, oeIIevent) == NS_OK)
	{ 
	  m_NumAttr++;
	}
    }
  RETURN( NS_OK );
}

NS_IMETHODIMP 
psuSunCalEvent::CreateRecurenceString(char **retString, 
				      oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::CreateRecurenceString()\n");
#endif
  PRUint32 interval;
  PRUint32 recurCount;
  char* recurUnits;
  if(( event->GetRecurInterval(&interval) !=  NS_OK) ||
     (event->GetRecurUnits(&recurUnits) != NS_OK) ||
     (event->GetRecurCount(&recurCount) != NS_OK))
    {
      RETURN( NS_ERROR_FAILURE );
    }
  char *DWMY;
  if(strcmp(recurUnits, "days") == 0)
    {
      DWMY = "D";
    }
  else if(strcmp(recurUnits, "weeks") == 0)
    {
      DWMY = "W";
    }
  else if(strcmp(recurUnits, "months") == 0)
    {
      DWMY = "MP";
    }
  else if((strcmp(recurUnits, "years") == 0))
    {
      DWMY = "YD";
    }
  else
    {
#ifdef PSUCAL_DEBUG
      printf("psuSunCalEvent::CreateRecurenceString : could not find recurence unit");
#endif
      RETURN( NS_ERROR_FAILURE );
    }
  (*retString) = (char *)nsMemory::Alloc(sizeof(char) * (5+(2*9)));
  if((*retString) == nsnull)
    {
      RETURN( NS_ERROR_OUT_OF_MEMORY );
    }
  PRBool repeatforever;
  event->GetRecurForever(&repeatforever);
  if(repeatforever)
    {
      /* value of 0 used to indicate infinite repeating */
      recurCount = 0;
    }
  
  
  /* setting reString using sprintf */
  sprintf((*retString),"%s%d #%d",DWMY,interval,recurCount);  
  printf("retString == %s\n", (*retString));
  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::CreateEmailReminder(CSA_reminder *retRem, oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::CreateMailReminder()\n");
#endif
  char *mailAddress;
  if(event->GetAlarmEmailAddress(&mailAddress) != NS_OK)
   {
    RETURN( NS_ERROR_FAILURE );
  }
  char *alarmUnit;
  PRUint32 alarmLength;
  if((event->GetAlarmUnits(&alarmUnit)) != NS_OK ||
    (event->GetAlarmLength(&alarmLength)) != NS_OK)
   {
     RETURN( NS_ERROR_FAILURE );
   }
  
  PRUint32 convLength;
  /* figuring lead time */
  /* have to convert the time to seconds */  
  if(strcmp(alarmUnit, "days") ==  0)
    {
      convLength = alarmLength * 24 * 60 * 60;
    }
  else if(strcmp(alarmUnit, "hours") == 0)
    {
      convLength = alarmLength * 60 * 60;
    }
  else if(strcmp(alarmUnit,"minutes")  == 0)
    {
      convLength = alarmLength * 60;
    }
  else 
    {
#ifdef PSUCAL_DEBUG
      printf("error didn't find the alarmUnit\n");
#endif
      RETURN( NS_ERROR_FAILURE );
    }
  char *lTString;
  /* 9 is the max length that a PRUint could be without overflowing */
  /* 4 is for the +PT and S in the string */
  lTString = (char*)nsMemory::Alloc(sizeof(char) * (4+9));
  if(lTString == nsnull)
    {
      RETURN( NS_ERROR_OUT_OF_MEMORY );
    }
/* creating string to handle event warning time in seconds */
  sprintf(lTString, "+PT%uS",  convLength); 
  retRem->lead_time = lTString;  
  retRem->reminder_data.data = (CSA_uint8*)mailAddress;
  retRem->reminder_data.size = 
    (CSA_uint32)(strlen(mailAddress));
  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::MakeDescription(char **retString, oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::MakeDescription()\n");
#endif
  nsCString title;
  nsCString description;
  nsCString location;
  if ((event->GetTitle(title) == NS_OK) &&
      (event->GetDescription(description) == NS_OK) &&
      (event->GetLocation(location) == NS_OK))
  {
    /* 2 is for number of ' ' in the string */
    PRInt32 textLength = 2;
    textLength += 
      title.Length() + description.Length() + location.Length();
    (*retString) =  (char *)nsMemory::Alloc(sizeof(char) * textLength);
    if(retString == nsnull)
      {
	RETURN( NS_ERROR_OUT_OF_MEMORY );
      }
    sprintf((*retString) ,"%s %s %s",
	    PromiseFlatCString(title).get(),
	    PromiseFlatCString(description).get(), 
	    PromiseFlatCString(location).get());
    RETURN( NS_OK );
  }
  RETURN( NS_ERROR_CAL_SERVER_INVALID_ATTRIBUTE_VALUE );
}

NS_IMETHODIMP 
psuSunCalEvent::OeDateTimeToIso8601(char **retString,  oeIDateTime *timeDate)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::OeDateTimeToIso8601() \n");
#endif
  PRInt16 year, month, day, hour, minute;

  if (timeDate == nsnull)
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }
  /* If any of these gets equal anything other then NS_OK then 
   * this will stop on that get and go to the code specified in the 
   * if statement */
  if( NS_FAILED(timeDate->GetYear(&year))
      || (NS_FAILED(timeDate->GetMonth(&month)))
      || (NS_FAILED(timeDate->GetDay(&day)))
      || (NS_FAILED(timeDate->GetHour(&hour)))
      || (NS_FAILED(timeDate->GetMinute(&minute))))
    {
      RETURN( NS_ERROR_CAL_SERVER_INVALID_ATTRIBUTE );
    }
 
  PRBool utcTime;
  timeDate->GetUtc(&utcTime);
  if(!utcTime)
    {
      /* adjusting to utc time or "Coordinated Universal Time" */
      hour += 4;      
    }
 
  PRInt32 stringLength = 17; 
  (*retString) = (char *)nsMemory::Alloc(sizeof(char) * stringLength);  
  if((*retString) == nsnull)
    {
      RETURN( NS_ERROR_OUT_OF_MEMORY );
    }
  sprintf((*retString), "%04d%02d%02dT%02d%02d00Z", 
	  year, month+1, day, hour, minute);
  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::SunCalEventToOeIICalEvent(oeIICalEvent **oeEvent)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::SunCalEventToOeIICalEvent()\n");
#endif
  oeIDateTime *newTime, *timeToSet;
  nsresult rv;
  RepeatInfo repeat;

  if( oeEvent == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }
  if( m_AttrList == nsnull || m_NumAttr == 0 )
    {
      RETURN( NS_ERROR_FAILURE );
    }

  nsCOMPtr<oeIICalEvent> comPEvent = do_CreateInstance(OE_ICALEVENT_CONTRACTID, &rv);
  if( NS_FAILED( rv ) )
    {
      RETURN( rv );
    }

  *oeEvent = comPEvent;
  NS_ADDREF(*oeEvent);

  for( PRInt32 i = 0; i < m_NumAttr; i++ )
    {
      char *name = m_AttrList[i].name;
      CSA_attribute_value *value = m_AttrList[i].value;
      /* A hash table to function pointers would be nice here */
      if( strcmp( name, CSA_ENTRY_ATTR_CLASSIFICATION ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrClassification( value, *oeEvent ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_ENTRY_ATTR_DATE_CREATED ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrDateCreated( value, *oeEvent ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_ENTRY_ATTR_END_DATE ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrEndDate( value, *oeEvent ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_ENTRY_ATTR_MAIL_REMINDER ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrMailReminder( value, *oeEvent ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_ENTRY_ATTR_START_DATE ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrStartDate( value, *oeEvent ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_ENTRY_ATTR_SUMMARY ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrSummary( value, *oeEvent ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_ENTRY_ATTR_TYPE ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrType( value, *oeEvent ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_X_DT_ENTRY_ATTR_REPEAT_TYPE ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrRepeatType( value, &repeat ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_X_DT_ENTRY_ATTR_REPEAT_TIMES ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrRepeatTimes( value, &repeat ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_X_DT_ENTRY_ATTR_REPEAT_INTERVAL ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrRepeatInterval( value, &repeat ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_X_DT_ENTRY_ATTR_REPEAT_OCCURRENCE_NUM ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrRepeatOccurrenceNum( value, &repeat ) ) )
            {
              RETURN( rv );
            }
        }
      else if( strcmp( name, CSA_X_DT_ENTRY_ATTR_SEQUENCE_END_DATE ) == 0 )
        {
          if( NS_FAILED( rv = DoAttrRepeatSequenceEndDate( value, &repeat ) ) )
            {
              RETURN( rv );
            }
        }
      /* there are other attributes, but they all fall into one of the following categories:
       *  1) oeICalEvent can't represent them.
       *  2) Purely informational, and I didn't want or couldn't use the information */
    }

  AssignRepeatInformation( *oeEvent, &repeat );

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::ExtractLeadTime(PRUint32 *leadTime, char *ltStr)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::ExtractLeadTime()\n");
#endif

  if ( ltStr == nsnull || leadTime == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  /* oeICal can't do negative alarm times, so we ignore the -PT case */
  if ( strncmp( ltStr, "+PT", strlen( "+PT" ) ) != 0 ) 
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

  char *nextTok;
  /* ltStr is converted to decimal format */
  PRUint32 result = (PRUint32)PR_strtod( &ltStr[4], &nextTok );

  if ( nextTok == nsnull || *nextTok != 'S' )
    {
#ifdef PSUCAL_DEBUG
      printf( "psuCalServerSunCDEImpl::ExtractLeadTime() "
              "- unimplemented lead time string [%s] encountered\n",
              ltStr );
#endif
      RETURN( NS_ERROR_INVALID_ARG );
    }

  *leadTime = result;
  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::Iso8601ToOeDateTime(oeIDateTime *oeEvent, char *iso8601)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::Iso8601ToOeDateTime()\n");
#endif

  nsresult rv;
  char *time = nsnull, *curField;
  PRInt16 sec, min, hour, day, month, year;

  if( oeEvent == nsnull || iso8601 == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if( strcmp( iso8601, FOREVER_DATE ) == 0 )
    {
      /* the forever date is a sentinel that is returned WAY too often from the 
       * csa server;  we can't rely on this to even really indicate that the event
       * was intended to recur forever, and it's also returned on events that don't
       * recurr at all.  Ignore. */
      return NS_OK;
    } 

  time = (char *)nsMemory::Clone( iso8601, strlen( iso8601 ) + 1 );

  /* time in format YYYYMMDDTHHMMSSZ
   * start from the back and work forward */
  sec = (PRInt16)PR_strtod( &time[13], NULL );
  time[13] = '\0';

  min = (PRInt16)PR_strtod( &time[11], NULL );
  time[11] = '\0';

  hour = (PRInt16)PR_strtod( &time[9], NULL );
  time[9] = '\0';

  day = (PRInt16)PR_strtod( &time[6], NULL );
  time[6] = '\0';

  month = (PRInt16)PR_strtod( &time[4], NULL );
  time[4] = '\0';

  year = (PRInt16)PR_strtod( time, NULL );
  nsMemory::Free( time );

  if( year < 1970 )
    {
#ifdef PSUCAL_DEBUG
      printf("psuSunCalEvent::Iso8601ToOeDateTime(): invalid date [%s] encountered\n", iso8601);
#endif
      return NS_OK;
    }

  if( NS_FAILED( rv = oeEvent->SetMinute( min ) )  ||
      NS_FAILED( rv = oeEvent->SetHour( hour ) )   ||
      NS_FAILED( rv = oeEvent->SetDay( day ) )     ||
      NS_FAILED( rv = oeEvent->SetMonth( month ) ) ||
      NS_FAILED( rv = oeEvent->SetYear( year ) ) )
    {
      RETURN( rv );
    }

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrClassification(CSA_attribute_value *value, oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrClassification()\n");
#endif
  nsresult rv;

  if ( value == nsnull || event == nsnull )
    {
      return NS_ERROR_NULL_POINTER;
    }

  if ( value->type != CSA_VALUE_UINT32 )
    {
      return NS_ERROR_INVALID_ARG;
    }

  /* there are more classifications, but oeICalEvent can't represent them, as this boolean should indicate */
  return event->SetPrivateEvent( value->item.uint32_value == CSA_CLASS_CONFIDENTIAL ? PR_TRUE : PR_FALSE );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrMailReminder(CSA_attribute_value *value, oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrMailReminder()\n");
#endif
  PRUint32 length, seconds;
  char *units;
  nsresult rv;

  if ( value == nsnull || event == nsnull )
    {
      return NS_ERROR_NULL_POINTER;
    }

  if ( value->type != CSA_VALUE_REMINDER )
    {
      return NS_ERROR_INVALID_ARG;
    }

  if ( NS_FAILED( rv = ExtractLeadTime( &seconds, value->item.reminder_value->lead_time ) ) )
    {
      RETURN( rv );
    }

  if ( seconds < 60 )
    {
      /* can't represent this with oeICalEventImpl */
      RETURN( NS_OK );
    }
  else if ( seconds / 60 < 60 )
    {
      length = seconds / 60;
      units = "minutes";
    }
  else if ( seconds / (60 * 60) < 24 )
    {
      length = seconds / (60 * 60);
      units = "hours";
    }
  else 
    {
      length = seconds / (24 * 60 * 60);
      units = "days";
    }
  char *email = (char *)nsMemory::Clone( value->item.reminder_value->reminder_data.data,
                                         value->item.reminder_value->reminder_data.size );
  if ( NS_FAILED( rv = event->SetAlarm( PR_TRUE ) ) ||
       NS_FAILED( rv = event->SetAlarmUnits( units ) ) ||
       NS_FAILED( rv = event->SetAlarmLength( length ) ) ||
       NS_FAILED( rv = event->SetAlarmEmailAddress( email ) ) )
    {
      nsMemory::Free( email );
      RETURN( rv );
    }
  nsMemory::Free( email );

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrStartDate(CSA_attribute_value *value, oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrStartDate()\n");
#endif
  oeIDateTime *startDate;
  nsresult rv;

  if ( value == nsnull || event == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }
  if ( value->type != CSA_VALUE_DATE_TIME )
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

  rv = event->GetStart( &startDate );
  if ( NS_FAILED( rv ) )
    {
      RETURN( rv );
    }

  rv = Iso8601ToOeDateTime( startDate, value->item.date_time_value );
  if ( NS_FAILED( rv ) )
    {
      RETURN( rv );
    }

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrEndDate(CSA_attribute_value *value, oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrEndDate()\n");
#endif
  oeIDateTime *endDate;
  nsresult rv;

  if ( value == nsnull || event == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if ( value->type != CSA_VALUE_DATE_TIME )
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

  rv = event->GetEnd( &endDate );
  if ( NS_FAILED( rv ) )
    {
      RETURN( rv );
    }

  rv = Iso8601ToOeDateTime( endDate, value->item.date_time_value );
  if ( NS_FAILED( rv ) )
    {
      RETURN( rv );
    }

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrSummary(CSA_attribute_value *value, oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrSummary()\n");
#endif
  nsresult rv;

  if ( value == nsnull || event == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if ( value->type != CSA_VALUE_STRING )
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

  nsCString description( value->item.string_value );
  if ( NS_FAILED( rv = event->SetDescription( description ) ) )
    {
      RETURN( rv );
    }

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrDateCreated(CSA_attribute_value *value, oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrDateCreated()\n");
#endif
  oeIDateTime *dateCreated;
  nsresult rv;

  if ( value == nsnull || event == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if ( value->type != CSA_VALUE_DATE_TIME )
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

  rv = event->GetStamp( &dateCreated );
  if ( NS_FAILED( rv ) )
    {
      RETURN( rv );
    }

  rv = Iso8601ToOeDateTime( dateCreated, value->item.date_time_value );
  if ( NS_FAILED( rv ) )
    {
      RETURN( rv );
    }

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrType(CSA_attribute_value *value, oeIICalEvent *event)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrType()\n");
#endif
  nsresult rv;

  if ( value == nsnull || event == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if ( value->type != CSA_VALUE_UINT32 )
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

#ifdef PSUCAL_DEBUG
  if ( value->item.uint32_value != CSA_TYPE_EVENT )
    {
      printf("unimplemented entry type [%d] encountered\n");
      /* We don't support non-events */
      return NS_OK;
    }
#endif

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrRepeatType(CSA_attribute_value *value, RepeatInfo *repeat)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrRepeatType()\n");
#endif
  nsresult rv;

  if ( value == nsnull || repeat == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if ( value->type != CSA_VALUE_SINT32 ) 
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

#ifdef PSUCAL_DEBUG
  printf( "Repeat Type: [%d] \n", value->item.sint32_value ); 
#endif

  repeat->type = value->item.sint32_value;

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrRepeatTimes(CSA_attribute_value *value, RepeatInfo *repeat)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrRepeatTimes()\n");
#endif
  nsresult rv;

  if ( value == nsnull || repeat == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if ( value->type != CSA_VALUE_UINT32 ) 
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

#ifdef PSUCAL_DEBUG
  printf( "repeat %d times\n", value->item.uint32_value );
#endif

  repeat->times = value->item.uint32_value;

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrRepeatInterval(CSA_attribute_value *value, RepeatInfo *repeat)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrRepeatInterval()\n");
#endif
  nsresult rv;

  if ( value == nsnull || repeat == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if ( value->type != CSA_VALUE_UINT32 ) 
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

#ifdef PSUCAL_DEBUG
  printf( "repeat interval: %d\n", value->item.uint32_value );
#endif

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrRepeatOccurrenceNum(CSA_attribute_value *value, RepeatInfo *repeat)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrRepeatOccurrenceNum()\n");
#endif
  nsresult rv;

  if ( value == nsnull || repeat == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if ( value->type != CSA_VALUE_SINT32 ) 
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

#ifdef PSUCAL_DEBUG
  printf( "repeat occur num: %d\n", value->item.sint32_value );
#endif
  repeat->occurnum = value->item.sint32_value;

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::DoAttrRepeatSequenceEndDate(CSA_attribute_value *value, RepeatInfo *repeat)
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::DoAttrRepeatSequenceEndDate()\n");
#endif
  nsresult rv;

  if ( value == nsnull || repeat == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  if ( value->type != CSA_VALUE_DATE_TIME ) 
    {
      RETURN( NS_ERROR_INVALID_ARG );
    }

#ifdef PSUCAL_DEBUG
  printf( "repeat end date [%s]\n", value->item.date_time_value );
#endif
  repeat->isodate = (char *)nsMemory::Clone( value->item.date_time_value, strlen( (char *)value->item.date_time_value ) );

  RETURN( NS_OK );
}

NS_IMETHODIMP psuSunCalEvent::AssignRepeatInformation( oeIICalEvent *event, RepeatInfo *repeat )
{
#ifdef PSUCAL_DEBUG
  printf("psuSunCalEvent::AssignRepeatInformation()\n");
#endif
  nsresult rv;
  char *units = nsnull;
  oeIDateTime *time;

  if ( event == nsnull || repeat == nsnull )
    {
      RETURN( NS_ERROR_NULL_POINTER );
    }

  switch ( repeat->type )
    {
    case CSA_X_DT_REPEAT_ONETIME:
      RETURN( event->SetRecur( PR_FALSE ) );
    case CSA_X_DT_REPEAT_DAILY:
      units = "days";
      break;
    case CSA_X_DT_REPEAT_WEEKLY:
      units = "weeks";
      break;
    case CSA_X_DT_REPEAT_MONTHLY_BY_DATE:
      units = "months_day";
      break;
    case CSA_X_DT_REPEAT_MONTHLY_BY_WEEKDAY:
      units = "months";
      break;
    case CSA_X_DT_REPEAT_YEARLY:
      units = "yearly";
      break;
    default:
      RETURN( NS_OK );
    }

  if ( NS_FAILED( rv = event->SetRecur( PR_TRUE ) ) ||
       NS_FAILED( rv = event->SetRecurForever( PR_FALSE ) ) ||
       NS_FAILED( rv = event->GetRecurEnd( &time ) ) ||
       NS_FAILED( rv = Iso8601ToOeDateTime( time, repeat->isodate ) ) ||
       NS_FAILED( rv = event->SetRecurInterval( repeat->interval ) ) ||
       NS_FAILED( rv = event->SetRecurUnits( units ) ) ||
       NS_FAILED( rv = event->SetRecurCount( repeat->times ) ) )
    {
      RETURN( rv );
    }

  RETURN( NS_OK );
}
